@@ -2,26 +2,28 @@
 !! Set of Fortran subroutines/functions for TARMA testing
 !! companion to the tseriesTARMA package
 !! Simone Giannerini
-!! 2021 - 
+!! 2020 - 
 ! *****************************************************************************
 
 ! *****************************************************************************
 
-SUBROUTINE TARMAGARCH(x,eps,h,n,d,trange,nr,p,ma,q,aa,m,bb,s,testv)
+SUBROUTINE TARMAGARCH(x,eps,h,n,d,trange,nr,p,ma,q,aa,m,bb,s,gyes,testv)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
  ! *****************************************************************************
- ! ARMA-GARCH vs TARMA-GARCH  supLM STATISTIC: test on both the AR and MA parameters
- ! SG 2021
+ ! ARMA-GARCH vs TARMA-GARCH  supLM STATISTIC: test on both the AR and MA parameters.
+ !  It includes the pure ARCH test  
+ ! SG 2024
  ! *****************************************************************************
-    INTEGER,intent(in):: n,nr,p,d,q,m,s
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),ma(q),aa(m),bb(s),h(n)
-    REAL(KIND=REAL64),intent(out):: testv(nr)
+    INTEGER,intent(in):: n,nr,p,d,q,m,s,gyes
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),ma(q),aa(m),bb(s),h(n)
+    REAL(C_DOUBLE),intent(out):: testv(nr)
     integer :: i,ii,k,neff,arlags(p+m+2),malags(q+m+2),glags(s+2)
-    REAL(KIND=REAL64) :: r,M11((p+q+1),(p+q+1)),M11i((p+q+1),(p+q+1)), M12((p+q+1),(p+q+1)), &
+    REAL(C_DOUBLE) :: r,M11((p+q+1),(p+q+1)),M11i((p+q+1),(p+q+1)), M12((p+q+1),(p+q+1)), &
     M21((p+q+1),(p+q+1)), M22((p+q+1),(p+q+1)), Mi((p+q+1),(p+q+1)), score(p+q+1), &
     scoreH0(p+q+1),scoreH1(p+q+1)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epsreg(:,:),hreg(:,:),epst(:), Xlag(:,:),elag(:,:),xth(:), &
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epsreg(:,:),hreg(:,:),epst(:), Xlag(:,:),elag(:,:),xth(:), &
     Xlagh(:,:),elagh(:,:),dedphi(:,:),dedth(:,:),dpsiar(:,:),dpsima(:,:),dedH0(:,:),ht(:), &
     dhdH0(:,:),dhdHdum(:,:),ones(:),Xdum(:,:),edum(:,:),hlag(:,:),eti(:), &
     dedphih(:,:),dedthh(:,:),dhdH02(:,:),dedH02(:,:),dhdH12(:,:),dedH12(:,:),dedH1(:,:),dhdH1(:,:)
@@ -81,7 +83,11 @@ SUBROUTINE TARMAGARCH(x,eps,h,n,d,trange,nr,p,ma,q,aa,m,bb,s,testv)
         dhdHdum = dhdHdum + 2*RESHAPE([dedphih,dedthh],[neff,(p+q+1)])*aa(i)*SPREAD(eti, DIM=2, NCOPIES=p+q+1) ! (neff * (p+q+1))
 !        CALL rfilter(eti**2,bb,s,neff,dhdai(:,i))        ! d(h_t)/d(a_i)
     enddo
-    CALL rfilterm(dhdHdum,bb,s,neff,(p+q+1),dhdH0)     ! # (neff * (p+q+1)) d(h_t)/d(Psi_1) partial derivatives w.r.t the ARMA parameters
+    if (gyes==1) then
+        CALL rfilterm(dhdHdum,bb,s,neff,(p+q+1),dhdH0)     ! # (neff * (p+q+1)) d(h_t)/d(Psi_1) partial derivatives w.r.t the 
+    else
+        dhdH0 = dhdHdum
+    endif 
     dedH02  = dedH0/SPREAD(sqrt(ht), DIM=2, NCOPIES=p+q+1)
     dhdH02  = dhdH0/SPREAD(ht, DIM=2, NCOPIES=p+q+1)
     M11     = (matmul(transpose(dedH02),dedH02)+0.5*matmul(transpose(dhdH02),dhdH02))/neff
@@ -121,71 +127,85 @@ SUBROUTINE TARMAGARCH(x,eps,h,n,d,trange,nr,p,ma,q,aa,m,bb,s,testv)
  END SUBROUTINE TARMAGARCH
 
 
+! *****************************************************************************
+
+
+
+
 ! *****************************************************************************
 
 ! *****************************************************************************
 
 SUBROUTINE ARvsTAR(x,eps,n,d,trange,nr,s2,p,testv)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
 ! *****************************************************************************
 ! ARvsTAR supLM STATISTICS
 ! SG 2020
 ! *****************************************************************************
     INTEGER,intent(in):: n,nr,p,d
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),s2
-    REAL(KIND=REAL64),intent(out):: testv(nr)
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),s2
+    REAL(C_DOUBLE),intent(out):: testv(nr,2)
     integer :: i,k,neff,lags(p+1)
-    REAL(KIND=REAL64) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
-    M22((p+1),(p+1)), Mi((p+1),(p+1)), score(p+1)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
+    REAL(C_DOUBLE) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
+    M22((p+1),(p+1)), Mi((p+1),(p+1)), score(p+1),scoreh(p+1),Vi(p+1,p+1)
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:),epstS(:,:),scoreht(:,:)
     INTEGER, ALLOCATABLE :: Ir(:)
     lags = [(i, i=1,p,1), 0]
     lags(p+1) = d
     call EMBED(x,lags,xreg)   ! X_{t} ...  X_{t-p},  X_{t-d} # (p+2) cols
     neff = SIZE(xreg, DIM=1)
     k    = MAX(p,d)
-    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),dphik(neff,(p+1)),dpsik(neff,(p+1)))
+    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),dphik(neff,(p+1)),dpsik(neff,(p+1)),&
+    epstS(neff,p+1),scoreht(neff,(p+1)))
     epst = eps((k+1):n)
+    epstS = SPREAD(epst, DIM=2, NCOPIES=(p+1))
     Xlag(:,1) = 1
     Xlag(:,2:(p+1)) = xreg(:,2:(p+1)) ! # 1, X_t-1, X_t-2 ... X_t-p
     xth  = xreg(:,(p+2))              ! # X_t-d threshold variable
 
     dphik = -Xlag                     ! neff * (p+1)
-    testv = 0
-    M11   = matmul(transpose(dphik),dphik)/(s2*neff)
+    testv(:,:) = 0
+    M11   = matmul(transpose(dphik),dphik)
     CALL INVERSE(M11,p+1,M11i)
     do i=1,nr,1 ! cycle over the threshold grid
         r  = trange(i)
         Ir = 0
         WHERE(xth <= r) Ir = 1 ! indicator function
         dpsik      = -Xlag*SPREAD(Ir, DIM=2, NCOPIES=(p+1)) ! neff * (p+1)
-        score      = -SUM(dpsik*SPREAD(epst, DIM=2, NCOPIES=(p+1)),DIM=1)/(sqrt(dble(neff))*s2) ! (p+1)
-        M22        = matmul(transpose(dpsik),dpsik)/(s2*neff)
-        M21        = matmul(transpose(dpsik),dphik)/(s2*neff)
-        M12        = transpose(M21)
-        CALL INVERSE(M22 - matmul(M21,matmul(M11i,M12)),p+1,Mi)
+        score      = -SUM(dpsik*SPREAD(epst, DIM=2, NCOPIES=(p+1)),DIM=1) ! (p+1)
+ !       M22        = matmul(transpose(dpsik),dpsik)
+        M21        = matmul(transpose(dpsik),dphik)
+!        M12        = transpose(M21)
+        CALL INVERSE(M21 - matmul(M21,matmul(M11i,M21)),p+1,Mi)
 !        testv(i)   = RESHAPE(matmul(transpose(score),matmul(Mi,score)),[1])
-        testv(i)   = dot_product(score,matmul(Mi,score))
+        testv(i,1)   = dot_product(score,matmul(Mi,score))/s2
+
+        scoreht = -transpose(dpsik*epstS) + matmul(matmul(M21,M11i),transpose(dphik*epstS))
+        scoreh = SUM(scoreht,DIM=2)
+        CALL INVERSE(matmul(scoreht,transpose(scoreht)),p+1,Vi)
+        testv(i,2)  = dot_product(scoreh,matmul(Vi,scoreh))
     enddo
  END SUBROUTINE ARvsTAR
 ! *****************************************************************************
 
 SUBROUTINE ARvsTARboot(x,eps,n,d,trange,nr,s2,p,testb,B)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
 ! *********************************************************************
 ! ARvsTAR supLM STATISTICS - bootstrap version (Hansen Econometrica 96)
 ! SG 2020
 ! *********************************************************************
     INTEGER,intent(in):: n,nr,p,d,B
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),s2
-    REAL(KIND=REAL64),intent(out):: testb(B)
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),s2
+    REAL(C_DOUBLE),intent(out):: testb(B)
     integer :: i,j,k,neff,lags(p+1)
-    REAL(KIND=REAL64) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
+    REAL(C_DOUBLE) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
     M22((p+1),(p+1)), Mi((p+1),(p+1)), scorepsi(p+1),scorephi(p+1),score(p+1),testv(nr)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
-    REAL(KIND=REAL64), ALLOCATABLE :: Ir(:),dum(:),eta(:,:)!,eta(:,:,:)
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
+    REAL(C_DOUBLE), ALLOCATABLE :: Ir(:),dum(:),eta(:,:)!,eta(:,:,:)
     lags = [(i, i=1,p,1), 0]
     lags(p+1) = d
     call EMBED(x,lags,xreg)   ! X_{t} ...  X_{t-p},  X_{t-d} # (p+2) cols
@@ -229,44 +249,52 @@ END SUBROUTINE ARvsTARboot
 
 SUBROUTINE ARMAvsTARMA(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
 ! *****************************************************************************
 ! ARMAvsTARMA supLM STATISTICS
 ! SG 2020
 ! *****************************************************************************
     INTEGER,intent(in):: n,nr,p,d,q
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
-    REAL(KIND=REAL64),intent(out):: testv(nr)
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
+    REAL(C_DOUBLE),intent(out):: testv(nr,2)
     integer :: i,k,neff,lags(p+1)
-    REAL(KIND=REAL64) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
-    M22((p+1),(p+1)), Mi((p+1),(p+1)), score(p+1)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
+    REAL(C_DOUBLE) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
+    M22((p+1),(p+1)), Mi((p+1),(p+1)), score(p+1),scoreh(p+1),Vi((p+1),(p+1))
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:),epstS(:,:),scoreht(:,:)
     INTEGER, ALLOCATABLE :: Ir(:)
     lags = [(i, i=1,p,1), 0]
     lags(p+1) = d
     call EMBED(x,lags,xreg)   ! X_{t} ...  X_{t-p},  X_{t-d} # (p+2) cols
     neff = SIZE(xreg, DIM=1)
     k    = MAX(p,d)
-    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),dphik(neff,(p+1)),dpsik(neff,(p+1)))
+    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),dphik(neff,(p+1)),dpsik(neff,(p+1)),&
+    scoreht(neff,(p+1)),epstS(neff,p+1))
     epst = eps((k+1):n)
+    epstS = SPREAD(epst, DIM=2, NCOPIES=(p+1))
     Xlag(:,1) = 1
     Xlag(:,2:(p+1)) = xreg(:,2:(p+1)) ! # 1, X_t-1, X_t-2 ... X_t-p
     xth  = xreg(:,(p+2))              ! # X_t-d threshold variable
-    CALL rfilterm(-Xlag,ma,q,neff,(p+1),dphik)
-    testv = 0
-    M11   = matmul(transpose(dphik),dphik)/(s2*neff)
+    CALL rfilterm(-Xlag,ma,q,neff,(p+1),dphik) ! d(eps_t)/d(phi_k)
+    testv(:,:) = 0
+    M11   = matmul(transpose(dphik),dphik)
     CALL INVERSE(M11,p+1,M11i)
     do i=1,nr,1 ! cycle over the threshold grid
         r  = trange(i)
         Ir = 0
         WHERE(xth <= r) Ir = 1 ! indicator function
-        CALL rfilterm(-Xlag*SPREAD(Ir, DIM=2, NCOPIES=(p+1)),ma,q,neff,(p+1),dpsik)
-        score      = -SUM(dpsik*SPREAD(epst, DIM=2, NCOPIES=(p+1)),DIM=1)/(sqrt(dble(neff))*s2) ! (p+1)
-        M22        = matmul(transpose(dpsik),dpsik)/(s2*neff)
-        M21        = matmul(transpose(dpsik),dphik)/(s2*neff)
+        CALL rfilterm(-Xlag*SPREAD(Ir, DIM=2, NCOPIES=(p+1)),ma,q,neff,(p+1),dpsik) ! d(eps_t)/d(Psi_k) k=0,2,..,p
+        score      = -SUM(dpsik*SPREAD(epst, DIM=2, NCOPIES=(p+1)),DIM=1) ! (p+1)
+        M22        = matmul(transpose(dpsik),dpsik)
+        M21        = matmul(transpose(dpsik),dphik)
         M12        = transpose(M21)
         CALL INVERSE(M22 - matmul(M21,matmul(M11i,M12)),p+1,Mi)
-        testv(i)   = dot_product(score,matmul(Mi,score))
+        testv(i,1)   = dot_product(score,matmul(Mi,score))/s2
+
+        scoreht = -transpose(dpsik*epstS) + matmul(matmul(M21,M11i),transpose(dphik*epstS))
+        scoreh = SUM(scoreht,DIM=2)
+        CALL INVERSE(matmul(scoreht,transpose(scoreht)),p+1,Vi)
+        testv(i,2)  = dot_product(scoreh,matmul(Vi,scoreh))
     enddo
  END SUBROUTINE ARMAvsTARMA
 
@@ -274,19 +302,20 @@ SUBROUTINE ARMAvsTARMA(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
 
 SUBROUTINE ARMAvsTARMAboot(x,eps,n,d,trange,nr,s2,p,ma,q,testb,B)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
 ! *********************************************************************
 ! ARMAvsTARMA supLM STATISTICS - bootstrap version (Hansen Econometrica 96)
 ! SG 2020
 ! *********************************************************************
     INTEGER,intent(in) :: n,nr,p,d,B,q
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
-    REAL(KIND=REAL64),intent(out):: testb(B)
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
+    REAL(C_DOUBLE),intent(out):: testb(B)
     integer :: i,j,k,neff,lags(p+1)
-    REAL(KIND=REAL64) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
+    REAL(C_DOUBLE) :: r,M11((p+1),(p+1)),M11i((p+1),(p+1)), M12((p+1),(p+1)), M21((p+1),(p+1)),&
     M22((p+1),(p+1)), Mi((p+1),(p+1)), scorepsi(p+1),scorephi(p+1),score(p+1),testv(nr)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
-    REAL(KIND=REAL64), ALLOCATABLE :: Ir(:),dum(:),eta(:,:)!,eta(:,:,:)
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epst(:),Xlag(:,:),xth(:),dphik(:,:),dpsik(:,:)
+    REAL(C_DOUBLE), ALLOCATABLE :: Ir(:),dum(:),eta(:,:),scoreht(:,:)
     lags = [(i, i=1,p,1), 0]
     lags(p+1) = d
     call EMBED(x,lags,xreg)   ! X_{t} ...  X_{t-p},  X_{t-d} # (p+2) cols
@@ -330,19 +359,20 @@ END SUBROUTINE ARMAvsTARMAboot
 
 SUBROUTINE ARMAvsTARMAg(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
     USE TARMA_MOD
+    use ISO_C_BINDING
     IMPLICIT NONE
 ! *****************************************************************************
 ! ARMAvsTARMAg supLM STATISTICS: test on both the AR and MA parameters
 ! SG 2021
 ! *****************************************************************************
     INTEGER,intent(in):: n,nr,p,d,q
-    REAL(KIND=REAL64),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
-    REAL(KIND=REAL64),intent(out):: testv(nr)
+    REAL(C_DOUBLE),intent(in) :: x(n),eps(n),trange(nr),s2,ma(q)
+    REAL(C_DOUBLE),intent(out):: testv(nr,2)
     integer :: i,k,neff,arlags(p+2),malags(q+2)
-    REAL(KIND=REAL64) :: r,M11((p+q+1),(p+q+1)),M11i((p+q+1),(p+q+1)), M12((p+q+1),(p+q+1)), M21((p+q+1),(p+q+1)),&
-    M22((p+q+1),(p+q+1)), Mi((p+q+1),(p+q+1)), score(p+q+1)
-    REAL(KIND=REAL64), ALLOCATABLE :: xreg(:,:),epsreg(:,:),epst(:),Xlag(:,:),epslag(:,:),xth(:),&
-    dphik(:,:),dths(:,:),dpsiar(:,:),dpsima(:,:),dedH0(:,:),dedH1(:,:)
+    REAL(C_DOUBLE) :: r,M11((p+q+1),(p+q+1)),M11i((p+q+1),(p+q+1)), M12((p+q+1),(p+q+1)), M21((p+q+1),(p+q+1)),&
+    M22((p+q+1),(p+q+1)), Mi((p+q+1),(p+q+1)), score(p+q+1), scoreh(p+q+1),Vi((p+q+1),(p+q+1))
+    REAL(C_DOUBLE), ALLOCATABLE :: xreg(:,:),epsreg(:,:),epst(:),Xlag(:,:),epslag(:,:),xth(:),&
+    dphik(:,:),dths(:,:),dpsiar(:,:),dpsima(:,:),dedH0(:,:),dedH1(:,:),epstS(:,:),scoreht(:,:)
     INTEGER, ALLOCATABLE :: Ir(:)
     k    = MAX(p,d,q)
     neff = n-k
@@ -354,8 +384,8 @@ SUBROUTINE ARMAvsTARMAg(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
     malags(q+2) = k
     call EMBED(x,arlags,xreg)     ! X_{t} ...     X_{t-p},   X_{t-d},   X_{t-k} # (p+3) cols
     call EMBED(eps,malags,epsreg) ! eps_{t} ... eps_{t-q}, eps_{t-d}, eps_{t-k} # (q+3) cols
-    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),epslag(neff,q),dphik(neff,(p+1)),&
-    dths(neff,q),dpsiar(neff,(p+1)),dpsima(neff,q),dedH0(neff,(p+q+1)),dedH1(neff,(p+q+1)))
+    ALLOCATE(epst(neff),xth(neff),Ir(neff),Xlag(neff,(p+1)),epslag(neff,q),dphik(neff,(p+1)),scoreht(neff,(p+q+1)),&
+    dths(neff,q),dpsiar(neff,(p+1)),dpsima(neff,q),dedH0(neff,(p+q+1)),dedH1(neff,(p+q+1)),epstS(neff,(p+q+1)))
     epst = epsreg(:,1)
     epslag(:,:) = epsreg(:,2:(q+1)) ! # eps_t-1, eps_t-2 ... eps_t-q
     Xlag(:,1) = 1
@@ -363,11 +393,12 @@ SUBROUTINE ARMAvsTARMAg(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
     xth  = xreg(:,(p+2))              ! # X_t-d threshold variable
     CALL rfilterm(-Xlag,ma,q,neff,(p+1),dphik) !# d(eps_t)/d(phi_k)
     CALL rfilterm(epslag,ma,q,neff,q,dths) !# d(eps_t)/d(theta_s)
-    testv = 0
+    testv(:,:) = 0
     dedH0(:,1:(p+1))       = dphik(:,:)
     dedH0(:,(p+2):(p+q+1)) = dths(:,:)
-    M11   = matmul(transpose(dedH0),dedH0)/(s2*neff)
+    M11   = matmul(transpose(dedH0),dedH0)
     CALL INVERSE(M11,p+q+1,M11i)
+    epstS = SPREAD(epst, DIM=2, NCOPIES=(p+q+1))
     do i=1,nr,1 ! cycle over the threshold grid
         r  = trange(i)
         Ir = 0
@@ -376,12 +407,16 @@ SUBROUTINE ARMAvsTARMAg(x,eps,n,d,trange,nr,s2,p,ma,q,testv)
         CALL rfilterm(epslag*SPREAD(Ir, DIM=2, NCOPIES=q),ma,q,neff,q,dpsima)         !# d(eps_t)/d(Psi_s) k=1,2,..,q
         dedH1(:,1:(p+1)) = dpsiar(:,:)
         dedH1(:,(p+2):(p+q+1)) = dpsima(:,:)
-        score      = -SUM(dedH1*SPREAD(epst, DIM=2, NCOPIES=(p+q+1)),DIM=1)/(sqrt(dble(neff))*s2) ! (p+q+1)
-        M22        = matmul(transpose(dedH1),dedH1)/(s2*neff)
-        M21        = matmul(transpose(dedH1),dedH0)/(s2*neff)
+        score      = -SUM(dedH1*epstS,DIM=1) ! (p+q+1)
+        M22        = matmul(transpose(dedH1),dedH1);
+        M21        = matmul(transpose(dedH1),dedH0);
         M12        = transpose(M21)
         CALL INVERSE(M22 - matmul(M21,matmul(M11i,M12)),p+q+1,Mi)
-        testv(i)   = dot_product(score,matmul(Mi,score))
+        testv(i,1)   = dot_product(score,matmul(Mi,score))/s2
+        scoreht = -transpose(dedH1*epstS) + matmul(matmul(M21,M11i),transpose(dedH0*epstS))
+        scoreh = SUM(scoreht,DIM=2)
+        CALL INVERSE(matmul(scoreht,transpose(scoreht)),p+q+1,Vi)
+        testv(i,2)  = dot_product(scoreh,matmul(Vi,scoreh))
     enddo
  END SUBROUTINE ARMAvsTARMAg
 
